// examples/smtpclient.cc
// This file is part of libpbe; see http://decimail.org
// (C) 2017 Philip Endecott

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include "SmtpClient.hh"
#include "strto.hh"

#include <stdlib.h>

#include <iostream>
#include <string>


static void usage()
{
  std::cerr << "Usage: smtpclient -h hostname    [localhost]\n"
               "                  -p portnum     [25]\n"
               "                  -d helo-domain\n"
               "                  -f from-address\n"
               "                  -t to-address\n"
               "Message headers and body are read from stdin.  LF is\n"
               "converted to CRLF.\n";
  exit(1);
}


int main (int argc, char* argv[])
{
  try { try {
    if (argc % 2 == 0) usage();  // Args are always in pairs
    
    std::string hostname = "localhost", portnum_s = "25",
                helo_domain, from_address, to_address;

    for (int a = 1; a < argc; a += 2) {
      std::string opt = argv[a];
      std::string val = argv[a+1];
      if      (opt == "-h") hostname     = val;
      else if (opt == "-p") portnum_s    = val;
      else if (opt == "-d") helo_domain  = val;
      else if (opt == "-f") from_address = val;
      else if (opt == "-t") to_address   = val;
      else throw "Unknown option " + opt;
    }

    if (hostname    .empty()) throw "hostname not specified";
    if (portnum_s   .empty()) throw "port number not specified";
    if (helo_domain .empty()) throw "helo domain not specified";
    if (from_address.empty()) throw "from address not specified";
    if (to_address  .empty()) throw "to address not specified";

    int portnum = pbe::strto<int>(portnum_s);

    SmtpClient smtpc;
    smtpc.connect(hostname,helo_domain,portnum);

    std::string message;
    while (std::cin.good()) {
      std::string line;
      getline(cin,line);
      message += line + "\r\n";
    }

    smtpc.send_msg(from_address,to_address,message);
    smtpc.disconnect();

    exit(0);

  } RETHROW_MISC_EXCEPTIONS }
  catch (pbe::Exception& E) {
    E.report(std::cout);
    exit(1);
  }
}

