// include/NumRange.hh
// This file is part of libpbe; see http://svn.chezphil.org/libpbe/
// (C) 2013 Philip Endecott

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef libpbe_NumRange_hh
#define libpbe_NumRange_hh

#include <boost/iterator/iterator_facade.hpp>


// NumRange is a container-like-class with a const_iterator that returns 
// a sequence of integers from 0 to n-1, where n is supplied to the NumRange ctor.


namespace pbe {


class NumRange
{
  int n;

public:
  class const_iterator: public boost::iterator_facade< const_iterator,
                                                       int,
                                                       std::forward_iterator_tag,
                                                       const int& >
  {
    int n;
    const int& dereference() const { return n; }
    bool equal(const const_iterator& other) const { return n == other.n; }
    void increment() { ++n; }
    friend class boost::iterator_core_access;
  public:
    const_iterator(int n_): n(n_) {}
  };

  NumRange(int n_): n(n_) {};

  const_iterator begin() const { return const_iterator(0); }
  const_iterator end()   const { return const_iterator(n); }
};


};


#endif

