// include/bstgmt.hh
// This file is part of libpbe; see http://svn.chezphil.org/libpbe/
// (C) 2017 Philip Endecott

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifndef bstgmt_hh
#define bstgmt_hh

#include <time.h>


namespace pbe {

// time_t is time since midnight 1st Jan 1970 UTC.  It ignores 
// time zones and summer time.  We normally prefer to work with 
// time_ts.  But time zones and summer time must be considered 
// during user I/O.
// 
// Here we assume that all times are in the UK, so we only need 
// to worry about BST/GMT.  To format a time_t for output, we 
// should show it in BST if it is a time when BST was in effect 
// and in GMT if not.

extern bool is_in_bst(time_t t);  // Is BST in effect at this time?

// This function takes a real time_t, i.e. in UTC, and applies 
// an offset if it is during a BST period.  The result is not 
// really a time_t any more, but it can be formatted for output.

extern time_t apply_offset_for_bst(time_t t);

// This function attempts to do the reverse of the above, but 
// this is ambiguous during the end-of-bst change over.

extern time_t remove_offset_for_bst(time_t t);

};


#endif

