// include/cpu_timer.hh
// This file is part of libpbe; see http://svn.chezphil.org/libpbe/
// (C) 2012 Philip Endecott

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef libpbe_cpu_timer_hh
#define libpbe_cpu_timer_hh

#include <limits>

#include <time.h>


namespace pbe {


class cpu_timer {
  clock_t start;
public:
  cpu_timer() {
    start = clock();
  }
  float operator()() const {
    // Avoid clock_t wraparound.
    int64_t now = clock();
    if (now < static_cast<int64_t>(start)) {
      now += std::numeric_limits<clock_t>::max();
    }
    return static_cast<float>(now-start) / CLOCKS_PER_SEC;
  }
};


};

#endif

