// include/elemsof.hh
// This file is part of libpbe; see http://svn.chezphil.org/libpbe/
// (C) 2014 Philip Endecott

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef libpbe_inc_dec_enum_hh
#define libpbe_inc_dec_enum_hh


// It's not normally possible to increment or decrement an enum 
// (though I seem to have got away with it for a long time thanks 
// to relaxed compilers).  But you can define operator++/-- for 
// the enum.
// 
// Of course this will only work if the enum values have contiguous 
// integer values.
// 
// The macro INC_DEC_ENUM can be used to define each of the two 
// versions of operator++/-- for you.  The definitions are inline, 
// so this can be used in a header.

// Recall that ++x returns the new value of x, and is declared with 
// the obvious prototype, while x++ returns the old value of x, and 
// is declared using the spare parameter.
 

#define INC_DEC_ENUM(E) \
inline E operator++(E& e) { int i = static_cast<int>(e); ++i; e = static_cast<E>(i); return e; } \
inline E operator--(E& e) { int i = static_cast<int>(e); --i; e = static_cast<E>(i); return e; } \
inline E operator++(E& e, int) { E ee = e; ++e; return ee; } \
inline E operator--(E& e, int) { E ee = e; --e; return ee; }



#endif
