// include/struct_compare.hh
// This file is part of libpbe; see http://svn.chezphil.org/libpbe/
// (C) 2013 Philip Endecott

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef libpbe_struct_compare_hh
#define libpbe_struct_compare_hh

// Macros to make it easier to write comparison functions for structs.
// Usage:
// 
// struct foo {
//   int a;
//   std::string b;
// };
// 
// bool operator<(const foo& a, const foo& b)
// {
//   FIELD_LESS(a)
//   FIELD_LESS(b)
//   return false;
// }
//
// bool operator==(const foo& a, const foo& b)
// {
//   FIELD_EQUAL(a)
//   FIELD_EQUAL(b)
//   return true;
// }


#define FIELD_LESS(fname) if (a.fname < b.fname) return true; if (b.fname < a.fname) return false;

#define FIELD_EQUAL(fname) if (a.fname != b.fname) return false;


#endif

